package model

import (
	"encoding/json"
	"time"
)

type Schematic struct {
	ID             string    `json:"id,omitempty" bson:"_id,omitempty"`
	Manufacturer   string    `json:"manufacturer" bson:"manufacturer,omitempty"`
	Model          string    `json:"model" bson:"model,omitempty"`
	SubTitle       string    `json:"subtitle" bson:"subtitle,omitempty"`
	Tags           []string  `json:"tags" bson:"tags,omitempty,inline"`
	Description    string    `json:"description" bson:"description,omitempty"`
	PrivateFile    bool      `json:"privateFile" bson:"privateFile,omitempty"`
	Owner          string    `json:"owner" bson:"Owner,omitempty"`
	Files          []string  `json:"files" bson:"files,omitempty,inline"`
	ForeignId      string    `json:"foreignId" bson:"foreignId,omitempty"`
	CreatedAt      time.Time `json:"createdAt" bson:"createdAt,omitempty"`
	LastModifiedAt time.Time `json:"lastModifiedAt" bson:"lastModifiedAt,omitempty"`
	BuildIn        time.Time `json:"buildIn" bson:"buildIn,omitempty"`
	BuildTO        time.Time `json:"buildTO" bson:"buildTO,omitempty"`
}

func (s *Schematic) UnmarshalJSON(data []byte) error {

	var dat map[string]interface{}
	if err := json.Unmarshal(data, &dat); err != nil {
		return err
	}

	s.ID = dat["id"].(string)
	if dat["manufacturer"] != nil {
		s.Manufacturer = dat["manufacturer"].(string)
	}

	if dat["model"] != nil {
		s.Model = dat["model"].(string)
	}

	if dat["subtitle"] != nil {
		s.SubTitle = dat["subtitle"].(string)
	}

	if dat["tags"] != nil {
		values := dat["tags"].([]interface{})
		s.Tags = make([]string, len(values))
		for i, d := range values {
			s.Tags[i] = d.(string)
		}
	}
	if dat["description"] != nil {
		s.Description = dat["description"].(string)
	}
	if dat["privateFile"] != nil {
		s.PrivateFile = dat["privateFile"].(float64) > 0
	}
	if dat["owner"] != nil {
		s.Owner = dat["owner"].(string)
	}
	if dat["files"] != nil {
		values := dat["files"].([]interface{})
		s.Files = make([]string, len(values))
		for i, d := range values {
			s.Files[i] = d.(string)
		}
	}
	if dat["foreignId"] != nil {
		s.ForeignId = dat["foreignId"].(string)
	}
	if dat["createdAt"] != nil {
		s.CreatedAt = time.Unix(0, int64(dat["createdAt"].(float64))*int64(time.Millisecond))
	}

	if dat["lastModifiedAt"] != nil {
		s.LastModifiedAt = time.Unix(0, int64(dat["lastModifiedAt"].(float64))*int64(time.Millisecond))
	}
	//	if dat["buildIn"] != nil {
	//		s.BuildIn = dat["buildIn"].(string)
	//	}
	//	if dat["buildTO"] != nil {
	//		s.BuildTO = dat["buildTO"].(string)
	//	}

	return nil
}
